<?php

  /*
    Plugin Name: Fanmail-Generated Posts
    Version: 0.0.1 alpha
    Description: Generate posts based on incoming fanmail via POP3
    Author: soulseekah
    Author URI: http://codeseekah.com
  */

    class FanmailPosts_Plugin {
        public function __construct() {
            add_action( 'init', array( $this, 'schedule_fetching' ) );
        }

        public function schedule_fetching() {
            if ( !wp_next_scheduled( 'fanmail_posts_run' ) ) {
                wp_schedule_event( time(), 'hourly', 'fanmail_posts_run' );
            }
        }

        public function run() {
            require_once ABSPATH . 'wp-admin/includes/taxonomy.php'; /* wp_create_category */

            $messages = $this->fetch_mail( 'ssl://pop.gmail.com', 995, 'you@yourdomain.com', 'strongpassword' );
            if ( is_wp_error( $messages ) ) return;
            
            if ( !$category = get_category_by_slug( 'fanmail' ) ) {
                $category_id = wp_create_category( 'Fan mail' );
            } else $category_id = $category->id;

            foreach( $messages as $message ) {
                /* create the posts */
                $content = htmlentities( $message['from'] ) . ' writes: '."\n";
                $content .= '<blockquote>' . strip_tags( $message['content'] ) . '</blockquote>';

                $post = array(
                        'post_category' => array( intval( $category_id ) ),
                        'post_content' => $content,
                        'post_status' => 'pending',
                        'post_title' => strip_tags( $message['subject'] ),
                        'post_type' => 'post'
                    );
                $post_id = wp_insert_post( $post );
            }
        }
    
        private function fetch_mail( $server, $port, $login, $pass ) {
            /* require POP3 library */
            require_once( ABSPATH . WPINC . '/class-pop3.php' );
       
            $pop = new POP3();
       
            /* invalid credentials */
            if ( !$pop->connect( $server, $port ) || !$pop->user( $login ) ) {
                return new WP_Error( 0x00, "POP Error: {$pop->ERROR}" );
            }

            /* no emails or invalid credentials */
            $mailcount = $pop->pass( $pass );
            if ( $mailcount === false ) return new WP_Error( 0x01, "POP Error: {$pop->ERROR}" );

            $messages = array(); /* store messages here */
      
            for ( $current = 1; $current <= $mailcount; $current++ ) {
         
                $body = false; $message = array();
                    foreach ( $pop->get( $current ) as $line ) {
                    if ( strlen( $line ) < 3 ) $body = true; /* a single \r\n */
                    if ( $body ) {
                        $message['content'] .= $line;
                    } else {
                        if (preg_match('/Subject: /i', $line)) {
                            $subject = trim($line);
                            $subject = substr($subject, 9); /* strip out Subject: */
                            $message['subject'] = $subject;
                        } elseif (preg_match('/From: /i', $line) )  {
                            $from = trim($line);
                            $from = substr($from, 6); /* strip out From */
                            $message['from'] = $from;
                        }
                    }
                } $messages []= $message; /* stack up on messages */
                $pop->delete( $current );
            } 
            return $messages;
        }
    }

    $fanmail_posts = new FanmailPosts_Plugin;
    add_action( 'fanmail_posts_run', array( $fanmail_posts, 'run' ) );
?>
